# -*- coding: utf8 -*-

"hikepy启动类"

__author__ = "hnck2000@126.com"

import os
import argparse

def main():
    "运行框架脚本"
     # 判断当前环境
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "arg1", help="命令", type=str, nargs="?", default="init"
    )
    args = parser.parse_args()
    current_path = os.getcwd()

    if args.arg1=="init":
        #项目脚手架生成
        print(current_path)
        #os.makedirs(os.path.join(root_path, "assets"), exist_ok=True)
        #os.makedirs(os.path.join(root_path, "assets", "etc"), exist_ok=True)
        #os.makedirs(os.path.join(root_path, "assets", "logs"), exist_ok=True)

if __name__ == "__main__":
    main()



# -*- coding: utf8 -*-

"hikepy全局配置类"

__author__ = "hnck2000@126.com"

import os
import time
from dataclasses import dataclass,field,asdict
import yaml
from .model import HkBaseData as baseData

@dataclass
class DBConfigData(baseData):
    """数据库配置类"""
    enable:bool=False
    url:str="mysql+aiomysql://user:password@localhost/dbname?charset=utf8mb4"
    pool_size:int=5
    max_overflow:int=10
    pool_timeout:int=30

@dataclass
class MogoConfigData(baseData):
    """MogoDB配置类"""
    enable:bool=False
    url:str="mongodb://admin:admin@127.0.0.1:27017"
    database:str="database"

@dataclass
class AppInfoData(baseData):
    """项目基本信息类"""
    title:str="hikepy生成项目"
    description:str="hikepy生成项目"
    summary:str="hikepy生成项目"
    version:str="0.0.1"
    contact:dict=field(default_factory=dict)

@dataclass
class RedisData(baseData):
    """缓存配置类"""
    enable:bool=False
    host:str="127.0.0.1"
    port:int=6379
    password:str=""
    db:int=0


@dataclass
class HkConfigData(baseData):
    """配置数据类"""
    env:str="dev"
    assets_path:str=field(default="",init=False)
    ip:str="0.0.0.0"
    port:int=8081
    appinfo:AppInfoData=field(default_factory=dict)
    cache:RedisData=field(default_factory=dict)
    database: DBConfigData=field(default_factory=dict)
    mogodb: MogoConfigData=field(default_factory=dict)
    custom:dict = field(default_factory=dict)

class HkConfig:
    """全局配置类"""
    SECRET_KEY:str = "3d78040d65bb4523da87bb71ba0728882a58e578fd88105a4a6d524e2735eeb6"
    ALGORITHM:str = "HS256"
    ACCESS_TOKEN_EXPIRE_DAYS:str = 7

    def __init__(self):
        self.data:HkConfigData=HkConfigData()
        self.data.appinfo=AppInfoData(contact={
        "name": "wantao",
        "url": "https://github.com/",
        "email": "hnck2000@126.com",
    })
        self.data.cache=RedisData()
        self.data.database=DBConfigData()
        self.data.mogodb=MogoConfigData()
        self.json:dict=asdict(self.data)
        self.sid:str=str(int(time.time() * 1000))
        self.id_generator=self.id_generator_func()

    def load_config(self,assets_path:str,host:str,port:int,env:str="dev"):
        """加载配置文件"""
        self.data.env=env
        self.data.assets_path=assets_path
        #优先使用命令行参数
        if host:
            self.data.ip=host
        if port:
            self.data.port=port
        
        #判断是否存在配置文件，如果不存在生成一个
        config_file_path=os.path.join(assets_path,"etc","config-"+env+".yaml")
        if os.path.exists(config_file_path):
            with open(config_file_path, 'r',encoding="utf-8") as f:
                self.json=yaml.safe_load(f)
                self.data=self.data.copy(self.json)
                self.data.assets_path=assets_path
        else:
            with open(config_file_path, 'w+',encoding="utf-8") as f:
                self.json=asdict(self.data)
                del self.json["assets_path"]
                yaml.safe_dump(self.json, f)

    def id_generator_func(self):
        """主键生成器"""
        n = 0
        while(True):
            n += 1
            yield n    

    def id(self,prex:str="00",start:int=0,nlen:int=10):
        """主键生成，生成规则prex-sid-id"""
        nmlen=10 if nlen<10 else nlen
        nb=start+next(config.id_generator)
        prex_s="" if prex=="" else prex+"-"
        str_num = str(nb).zfill(nmlen)
        return f"{prex_s}{config.sid}-{str_num}"

config=HkConfig()



# -*- coding: utf8 -*-

"hikepy工具类"

__author__ = "hnck2000@126.com"
import os
import pkgutil
import importlib
import inspect
from datetime import datetime,timedelta
import redis
import jwt
import httpx
import orjson
from typing import Any
from redis import ConnectionPool, Redis, RedisError
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.ext.asyncio import (
    AsyncSession,
    create_async_engine,
    AsyncEngine,
    async_sessionmaker,
)
from sqlalchemy.engine.interfaces import _CoreAnyExecuteParams
from sqlalchemy.orm import sessionmaker
from sqlalchemy import text, Executable,func,select
from motor.motor_asyncio import AsyncIOMotorClient,AsyncIOMotorDatabase

from .hklogger import hklog
from .model import SqlModel,PageList,AuthInfo
from .config import config

def create_token(data:AuthInfo,days:int=config.ACCESS_TOKEN_EXPIRE_DAYS)->str:
    """生成认证token"""
    pdict=data.model_dump()
    pdict["exp"]=datetime.now() + timedelta(days=days)
    return jwt.encode(pdict, config.SECRET_KEY, algorithm=config.ALGORITHM)

class HkCache:
    """缓存客户端工具栏"""

    def __init__(self):
        self.pool: ConnectionPool = None

    def __del__(self):
        if self.pool:
            self.pool.disconnect()

    def init(self, host: str, port: int, password: str, decode_responses=True,db:int=0):
        """缓存连接池初始化"""
        try:
            self.pool = redis.ConnectionPool(
                host=host,
                port=port,
                password=password,
                db=db,
                decode_responses=decode_responses
            )
        except RedisError as ext:
            hklog.error(f"cache init faild error:{str(ext)}")

    def client(self) -> Redis:
        """获取redis客户端"""
        if self.pool:
            return redis.Redis(connection_pool=self.pool)


hkcache = HkCache()


class HkDatabaseManager:
    """数据库连接管理类"""

    def __init__(self):
        self.engine: AsyncEngine = None
        self.async_session_factory: sessionmaker = None
        self.models: list[SqlModel] = []

    # def __del__(self):
    #     if self.engine is not None:
    #         self.engine.dispose()

    def init(self, url: str, pool_size: int = 5, max_overflow=10, pool_timeout=30):
        """数据库初始化"""
        try:
            self.engine = create_async_engine(
                url,
                echo=True,
                future=True,
                pool_size=pool_size,
                max_overflow=max_overflow,
                pool_timeout=pool_timeout,
            )
            self.async_session_factory = async_sessionmaker(
                self.engine, expire_on_commit=False, class_=AsyncSession
            )
        except SQLAlchemyError as ext:
            hklog.error(f"database init faild error:{str(ext)}")

    def create_session(self) -> AsyncSession:
        """获取sql工具类"""
        return self.async_session_factory()

    def scan_models(self, model_root: str, pkg_path: str):
        """扫描指定包下的路由文件"""
        for module in pkgutil.walk_packages([model_root], pkg_path + "."):
            if module.ispkg:
                self.scan_models(os.path.join(model_root, module.name), module.name)
            else:
                if module.name.split(".")[-1] == "models":
                    module = importlib.import_module(module.name)
                    for name, value in module.__dict__.items():
                        if (
                            name != "SqlModel"
                            and inspect.isclass(value)
                            and issubclass(value, SqlModel)
                        ):
                            self.models.append(value)

          

    async def register_database(self, model_root: str, pkg_path: str):
        """初始化创建数据库"""
        if self.engine:
            self.scan_models(model_root, pkg_path)
            async with self.engine.begin() as conn:
                for md in self.models:
                    await conn.run_sync(md.__table__.create, checkfirst=True)


hk_db_manager = HkDatabaseManager()

class HkMongoManager:
    """Mongo连接管理类"""
    def __init__(self):
        self.client:AsyncIOMotorClient=None
        self.db:Mogo=None
        self.database:str=""

    def __del__(self):
        if self.client is not None:
            self.client.close()

    def init(self, url: str,database:str):
        """mogo初始化"""
        try:
            self.client= AsyncIOMotorClient(url)
            self.database=database
            self.db=self.client[database]
        except Exception as error:
            hklog.error(f"mogo init faild error:{str(error)}")




hk_mogo_manager = HkMongoManager()

class Sql:
    """数据库操作工具类"""

    def __init__(self, session: AsyncSession):
        self.session = session
        hklog.info("reject session width autocommit")

    async def exec(self, sql: str, param: dict | None = None):
        """执行原生SQL"""
        return await self.session.execute(text(sql), param)

    async def execute(
        self, statement: Executable, params: _CoreAnyExecuteParams | None = None
    ):
        """execute"""
        return await self.session.execute(statement, params)

    async def scalars(
        self, statement: Executable, params: _CoreAnyExecuteParams | None = None
    ):
        """scalars"""
        return await self.session.scalars(statement, params)

    async def scalar(
        self, statement: Executable, params: _CoreAnyExecuteParams | None = None
    ):
        """scalars"""
        return await self.session.scalar(statement, params)

    async def get(self, *args, **kwargs):
        """get"""
        return await self.session.get(*args, **kwargs)

    async def add(self, instance: object, _warn: bool = True):
        """add"""
        return self.session.add(instance, _warn)

    async def delete(self, instance: object):
        """add"""
        return await self.session.delete(instance)

    async def merge(self, *args, **kwargs):
        """merge"""
        return await self.session.merge(*args, **kwargs)

    async def run_sync(self, *args, **kwargs):
        """run_sync"""
        return await self.session.run_sync(*args, **kwargs)
    
    async def page(self, statement: Executable, params: _CoreAnyExecuteParams | None = None,current_page:int=1,page_size:int=20):
        """分页查询"""
        if statement.is_select:
            total_count=await self.session.scalar(select(func.count("*")).select_from(statement),params)
            result=await self.execute(statement.offset((current_page - 1) * page_size).limit(page_size), params)
            result_dict=[]
            for row in result.all():
                row_dict=row._asdict()
                if len(row_dict.keys()) > 1:
                    result_row={}
                    for r_key in row_dict.keys():
                        if isinstance(row_dict[r_key], SqlModel):
                            result_row[r_key]=row_dict[r_key].model_dump()
                        else:
                            result_row[r_key]=row_dict[r_key]
                    result_dict.append(result_row)    
                else:
                    if isinstance(row[0], SqlModel):
                        result_dict.append(row[0].model_dump())
                    else:
                        result_dict.append(row[0])
            return PageList.create(datas=result_dict,current_page=current_page,total_size=total_count,page_size=page_size)
        else:
            return PageList.create([])

    async def query(self, statement: Executable, params: _CoreAnyExecuteParams | None = None):
        """分页查询"""
        if statement.is_select:
            result=await self.execute(statement, params)
            result_dict=[]
            for row in result.all():
                row_dict=row._asdict()
                if len(row_dict.keys()) > 1:
                    result_row={}
                    for r_key in row_dict.keys():
                        if isinstance(row_dict[r_key], SqlModel):
                            result_row[r_key]=row_dict[r_key].model_dump()
                        else:
                            result_row[r_key]=row_dict[r_key]
                    result_dict.append(result_row)    
                else:
                    if isinstance(row[0], SqlModel):
                        result_dict.append(row[0].model_dump())
                    else:
                        result_dict.append(row[0])
            return result_dict
        else:
            return []    
        

class Mogo(AsyncIOMotorDatabase):
    """Mogodb操作工具类"""

class Rest:
    """restapi操作工具类"""
    def __init__(self, client: httpx.AsyncClient):
        self.client = client
        self.client.timeout = httpx.Timeout(3, connect=5, read=5, write=5)
        self.client.headers["User-Agent"] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36"

    async def close(self):
        """关闭连接"""
        await self.client.aclose()

    async def get(self, url: str, params: dict | None = None,token:str|None=None,header:httpx._types.HeaderTypes|None=None,timeout:int|None=None)->dict:
        """get请求"""
        result={"code": 200, "message": "", "data": {}}
        t_headers = self.client.headers.copy()
        if token:
            t_headers["token"] = token
        if header:
            t_headers.update(header)
        if timeout and timeout > 0:
            timeout1 = httpx.Timeout(3, connect=5, read=timeout, write=timeout)    
        try:
            response = await self.client.get(url, params=params,headers=t_headers,timeout=timeout1)
            result=orjson.loads(response.content)
        except httpx.HTTPError as ext:
            result["code"] = 500
            result["message"] = str(ext)
        return result
    
    async def post(self, url: str, params: dict | None = None,json:Any|None=None,token:str|None=None,header:httpx._types.HeaderTypes|None=None,timeout:int|None=None)->dict:
        """post请求"""
        result={"code": 200, "message": "", "data": {}}
        t_headers = self.client.headers.copy()
        if token:
            t_headers["token"] = token
        if header:
            t_headers.update(header)
        if timeout and timeout > 0:
            timeout1 = httpx.Timeout(3, connect=5, read=timeout, write=timeout)    
        try:
            response = await self.client.post(url, params=params,headers=t_headers,json=json,timeout=timeout1)
            result = orjson.loads(response.content)
        except httpx.HTTPError as ext:
            result["code"] = 500
            result["message"] = str(ext)
        return result



# -*- coding: utf8 -*-

"hikepy启动类"

__author__ = "hnck2000@126.com"

import os
import inspect
import argparse
import pkgutil
import importlib
import json
import time
from typing import List, Annotated, Mapping, Any
import uvicorn
import jwt
from jwt.exceptions import InvalidTokenError
from fastapi import FastAPI, Request, status, APIRouter, HTTPException, Header
from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse, Response
from fastapi.middleware.cors import CORSMiddleware
from starlette.exceptions import HTTPException as StarletteHTTPException
from starlette.background import BackgroundTask
from pydantic import ValidationError
from .hklogger import hklog
from .tools import hkcache, hk_db_manager, hk_mogo_manager
from .config import config
from .business import HkBusinessException
from .model import AuthInfo



# 定义一个异步函数的类型提示
class HkJSONResponse(Response):
    """自定义响应类"""

    media_type = "application/json"

    def __init__(
        self,
        content: Any,
        status_code: int = 200,
        headers: Mapping[str, str] | None = None,
        media_type: str | None = None,
        background: BackgroundTask | None = None,
    ) -> None:
        super().__init__(content, status_code, headers, media_type, background)

    def render(self, content: Any) -> bytes:
        newcontent = {"code": self.status_code, "message": "", "data": content}
        result = json.dumps(
            newcontent,
            ensure_ascii=False,
            allow_nan=False,
            indent=None,
            separators=(",", ":"),
        ).encode("utf-8")
        return result


class HkAuth:
    """jwt验证类"""

    def __init__(self, role: List[str] = None, act: List[str] = None):
        self.role = role
        self.act = act

    def __call__(
        self, token: Annotated[str | None, Header(description="auth token")] = None
    ) -> AuthInfo:
        credentials_exception=HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail="Not enough permissions",
            )

        if token is None:
            raise credentials_exception
        try:
            payload = jwt.decode(token, config.SECRET_KEY, algorithms=[config.ALGORITHM])
            user_id: str = payload.get("user_id")
            user_roles: List[str]=payload.get("roles")
            user_acts: List[str]=payload.get("acts")
            data: dict = payload.get("data")
            if user_id is None:
                raise credentials_exception
            r_checked=False
            a_checked=False
            if self.role:
                if user_roles:
                    for r1 in self.role:
                        if r1 in user_roles:
                            r_checked=True
            else:
                r_checked=True

            if self.act:
                if user_acts:
                    for r1 in self.act:
                        if r1 in user_acts:
                            a_checked=True
            else:
                a_checked=True

            if not (r_checked and a_checked):
                raise credentials_exception
            return AuthInfo(user_id=user_id,data=data,roles=user_roles,acts=user_acts)
        except (InvalidTokenError, ValidationError) as exc:
            print(str(exc))
            raise credentials_exception from exc


class HkApp:
    """启动类"""

    def __init__(self):
        self.app: FastAPI = None
        self.prehook_caller = None
        self.routers: list[APIRouter] = []

    def init_assets(self, root_path: str):
        """初始化资源结构"""
        os.makedirs(os.path.join(root_path, "assets"), exist_ok=True)
        os.makedirs(os.path.join(root_path, "assets", "etc"), exist_ok=True)
        os.makedirs(os.path.join(root_path, "assets", "logs"), exist_ok=True)

    def prehook(self, func):
        """初始化钩子装饰器"""

        async def wrapper(*args, **kwargs):
            return await func(*args, **kwargs)

        self.prehook_caller = wrapper
        return wrapper

    def get_fastapi_logconfig(self) -> dict:
        """设置fastapi日志格式及路径"""
        log_path = os.path.join(config.data.assets_path, "logs", "access.log")
        server_log_path = os.path.join(config.data.assets_path, "logs", "server.log")
        return {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "default": {
                    "()": "uvicorn.logging.DefaultFormatter",
                    "fmt": "[%(asctime)s][%(levelname)s]: %(message)s",
                    "use_colors": None,
                },
                "access": {
                    "()": "uvicorn.logging.AccessFormatter",
                    "fmt": '[%(asctime)s][%(levelname)s]: %(client_addr)s - "%(request_line)s" %(status_code)s',
                },
            },
            "handlers": {
                "default": {
                    "formatter": "default",
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stderr",
                },
                "access": {
                    "formatter": "access",
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stdout",
                },
                "default_file": {
                    "formatter": "default",
                    "class": "logging.handlers.RotatingFileHandler",
                    "filename": server_log_path,
                },
                "access_file": {
                    "formatter": "access",
                    "class": "logging.handlers.TimedRotatingFileHandler",
                    "filename": log_path,
                },
            },
            "loggers": {
                "uvicorn": {"handlers": ["default", "default_file"], "level": "INFO"},
                "uvicorn.error": {"level": "INFO"},
                "uvicorn.access": {
                    "handlers": ["access_file"],
                    "level": "INFO",
                    "propagate": False,
                },
            },
        }

    def scan_routers(self, router_root: str, pkg_path: str):
        """扫描指定包下的路由文件"""
        for module in pkgutil.walk_packages([router_root], pkg_path + "."):
            if module.ispkg:
                self.scan_routers(os.path.join(router_root, module.name), module.name)
            else:
                if module.name.split(".")[-1] == "routers":
                    router = importlib.import_module(module.name)
                    for name, value in router.__dict__.items():
                        if name == "router" and isinstance(value, APIRouter):
                            self.routers.append(value)

    def scan_service(self, service_root: str, pkg_path: str):
        """扫描指定包下的路由文件"""
        for module in pkgutil.walk_packages([service_root], pkg_path + "."):
            if module.ispkg:
                self.scan_service(os.path.join(service_root, module.name), module.name)
            else:
                if module.name.split(".")[-1] == "service":
                    importlib.import_module(module.name)

    def get_startup(self, root_path: str, router_root: str):
        """定义fastapi初始化事件"""
        self.scan_service(
            os.path.join(root_path, router_root.replace(".", os.sep)), router_root
        )

        async def start_app():
            # 创建数据库
            await hk_db_manager.register_database(
                os.path.join(root_path, router_root.replace(".", os.sep)), router_root
            )
            # 留给研发进行扩展启动方法
            if self.prehook_caller:
                await self.prehook_caller()
                hklog.info("prehook exec completed")
            else:
                hklog.info("prehook not set")

        return start_app

    def start(self, autostar: bool = True, router_root: str = "app.modules"):
        """开启服务"""
        print("server init....")
        caller_file = inspect.getfile(inspect.currentframe().f_back)
        root_path = os.path.dirname(caller_file)
        # 判断当前环境
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "arg1", help="应用当前环境", type=str, nargs="?", default="dev"
        )
        parser.add_argument("--host", help="IP地址", type=str, required=False)
        parser.add_argument("--port", help="监听端口", type=int, required=False)

        args = parser.parse_args()
        # 初始化资源目录
        print("server assets init......")
        self.init_assets(root_path)
        print("server config init......")
        # 初始化全局配置
        config.load_config(
            os.path.join(root_path, "assets"), args.host, args.port, args.arg1
        )
        hklog.info("env check completed,current env is " + args.arg1)
        hklog.info("assets check completed")
        hklog.info("config load completed")
        hklog.debug(config.data.__dict__)

        # 业务工具初始化
        if "cache" in config.json and config.json["cache"]["enable"]:
            hkcache.init(
                host=config.json["cache"]["host"],
                port=config.json["cache"]["port"],
                password=config.json["cache"]["password"],
                decode_responses=True,
                db=config.json["cache"]["db"]
            )
        if "database" in config.json and config.json["database"]["enable"]:
            hk_db_manager.init(
                url=config.json["database"]["url"],
                pool_size=config.json["database"]["pool_size"],
                max_overflow=config.json["database"]["max_overflow"],
                pool_timeout=config.json["database"]["pool_timeout"],
            )

        if "mogodb" in config.json and config.json["mogodb"]["enable"]:
            hk_mogo_manager.init(
                config.json["mogodb"]["url"], config.json["mogodb"]["database"]
            )

        self.app = FastAPI(
            default_response_class=HkJSONResponse,
            title=config.json["appinfo"]["title"],
            description=config.json["appinfo"]["description"],
            summary=config.json["appinfo"]["summary"],
            version=config.json["appinfo"]["version"],
            contact=config.json["appinfo"]["contact"],
            Response="",
            openapi_url=None,
            docs_url=None,
            redoc_url=None
        )

        # 自定义异常处理
        @self.app.exception_handler(HkBusinessException)
        async def business_exception_handler(
            request: Request, exc: HkBusinessException
        ):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content={"code": exc.code, "message": exc.message},
            )

        @self.app.exception_handler(StarletteHTTPException)
        async def http_exception_handler(request: Request, exc: StarletteHTTPException):
            return JSONResponse(
                status_code=exc.status_code,
                content={"code": exc.status_code, "message": str(exc.detail)},
            )

        @self.app.exception_handler(HTTPException)
        async def http_exception_fastapi_handler(request: Request, exc: HTTPException):
            return JSONResponse(
                status_code=exc.status_code,
                content={"code": exc.status_code, "message": str(exc.detail)},
            )

        @self.app.exception_handler(ValidationError)
        async def pydict_validation_exception_handler(
            request: Request, exc: ValidationError
        ):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content={
                    "code": status.HTTP_400_BAD_REQUEST,
                    "message": "错误的请求参数",
                },
            )

        @self.app.exception_handler(RequestValidationError)
        async def validation_exception_handler(
            request: Request, exc: RequestValidationError
        ):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content={
                    "code": status.HTTP_400_BAD_REQUEST,
                    "message": str(exc),
                    "body": exc.body,
                },
            )

        # 跨域中间件
        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )

        # 自定义中间件
        @self.app.middleware("http")
        async def add_process_time_header(request: Request, call_next):
            start_time = time.perf_counter()
            response = await call_next(request)
            process_time = round(time.perf_counter() - start_time, 4)
            response.headers["X-Process-Time"] = str(process_time)
            hklog.info("URL:" + request.url.path + " time:" + str(process_time))
            return response

        # 自定义全局注入

        # 添加自定义事件
        self.app.add_event_handler("startup", self.get_startup(root_path, router_root))

        # 扫描路径自动加载路由
        self.scan_routers(
            os.path.join(root_path, router_root.replace(".", os.sep)), router_root
        )
        for router in self.routers:
            self.app.include_router(router)

        # 开启fastapi服务
        hklog.info("server started")
        if autostar:
            log_config = self.get_fastapi_logconfig()
            uvicorn.run(
                self.app,
                host=config.data.ip,
                port=config.data.port,
                log_config=log_config,
            )


hkapp = HkApp()



# -*- coding: utf8 -*-

"hikepy日志记录类"

__author__ = "hnck2000@126.com"

import os
import logging
from logging import Logger
from logging.handlers import RotatingFileHandler
from .config import config

class HkLogger:
    """日志记录类"""

    def __init__(self):
        self.loggers:dict[str,Logger]={}

    def get_logger(self,name:str)->Logger:
        """根据名称获取日志记录类"""
        if name in self.loggers:
            return self.loggers[name]
        else:
            new_logger = logging.getLogger(name)
            formatter = logging.Formatter("[%(asctime)s][%(levelname)s]: %(message)s")
            if config.data.env=="dev":
                new_logger.setLevel(logging.DEBUG)
            else:
                new_logger.setLevel(logging.INFO)

            # 创建控制台处理器
            console_handler = logging.StreamHandler()
            console_handler.setFormatter(formatter)
            # 创建文件处理器
            file_handler = RotatingFileHandler(os.path.join(config.data.assets_path,"logs", name+'.log'),maxBytes=5*1024*1024, backupCount=1)
            file_handler.setFormatter(formatter)

            # 将处理器添加到logger
            new_logger.addHandler(console_handler)
            new_logger.addHandler(file_handler)

            self.loggers[name]=new_logger
            return new_logger


    def set_env(self,level:int=logging.INFO,name:str="app"):
        """设置日志记录级别"""
        logger=self.get_logger(name)
        logger.setLevel(level)

    def debug(self,msg:str,name:str="app"):
        """debug"""
        logger=self.get_logger(name)
        logger.debug(msg)

    def info(self,msg:str,name:str="app"):
        """info"""
        logger=self.get_logger(name)
        logger.info(msg)

    def warning(self,msg:str,name:str="app"):
        """warning"""
        logger=self.get_logger(name)
        logger.warning(msg)

    def error(self,msg:str,name:str="app"):
        """error"""
        logger=self.get_logger(name)
        logger.error(msg)

    def critical(self,msg:str,name:str="app"):
        """critical"""
        logger=self.get_logger(name)
        logger.critical(msg)



hklog=HkLogger()



"""
hikepy framework
"""

from .app import hkapp,HkAuth
from .hklogger import hklog
from .tools import hkcache
from .tools import hk_db_manager,hk_mogo_manager,create_token
from .config import config
from .model import HkBaseData as baseData,HkBaseModel as baseModel,SqlModel,PageList,AuthInfo,CommonModel,MogoModel
from .business import HkBusinessException,service

__version__ = '1.0.0'

__all__ = [
    '__version__',
    'hkapp',
    'hklog',
    'config',
    'baseData',
    'baseModel',
    'HkBusinessException',
    'service',
    'hkcache',
    'hk_db_manager',
    'SqlModel',
    'MogoModel',
    'PageList',
    'hk_mogo_manager',
    'AuthInfo',
    'HkAuth',
    'create_token',
    'CommonModel'
]

def __dir__() -> list[str]:
    return __all__



# -*- coding: utf8 -*-

"hikepy业务核心类"

__author__ = "hnck2000@126.com"

import time
import inspect
import httpx
from redis import Redis

# from typing import Annotated,get_args
from .hklogger import hklog
from .tools import hkcache, Sql, hk_db_manager,hk_mogo_manager,Mogo,Rest


class HkBusinessException(Exception):
    """自定义业务异常"""

    def __init__(self, code: int = 0, message: str = ""):
        """构造函数"""
        self.code = code
        self.message = message

def service(func):
    """业务方法装饰器"""

    async def wrapper(*args, **kwargs):
        # annotations = func.__annotations__
        func_sig = inspect.signature(func)
        start_time = time.perf_counter()
        func_file = inspect.getfile(func)
        sql_session = None
        cache_inst = None
        rest_client = None
        in_transcation=False
        hklog.info("enter service:" + func.__name__ + " file:" + func_file)
        try:
            for _, param in func_sig.parameters.items():
                if param.annotation != inspect.Parameter.empty:
                    arg_class = None
                    # arg_meta=None
                    # anno_meta=get_args(param.annotation)
                    # if len(anno_meta)>0:
                    #    arg_class=anno_meta[0]
                    #    arg_meta=anno_meta[1]
                    # else:
                    #    arg_class=param.annotation
                    arg_class = param.annotation
                    if issubclass(arg_class, Sql):
                        # 数据库工具装入
                        # if anno_meta:
                        #    sql_inst=arg_class(**arg_meta.__dict__)
                        # else:
                        #    sql_inst=arg_class()
                        if kwargs.get(param.name):
                            sql_session=kwargs[param.name]
                            in_transcation=True
                        
                        if sql_session is None:
                            sql_session = hk_db_manager.create_session()
                            await sql_session.begin()
                            kwargs[param.name] = Sql(sql_session)
                    elif issubclass(arg_class, Redis):
                        # Cache工具装入
                        cache_inst = hkcache.client()
                        kwargs[param.name] = cache_inst
                    elif issubclass(arg_class, Mogo):
                        # mogo工具装入
                        kwargs[param.name] = hk_mogo_manager.db
                    elif issubclass(arg_class, Rest):
                        # rest工具装入
                        rest_client = httpx.AsyncClient()
                        kwargs[param.name] = Rest(rest_client)   

            result = await func(*args, **kwargs)
            if sql_session and in_transcation is False:
                await sql_session.commit()
        except HkBusinessException as e:
            hklog.error(f"raise business code:{e.code} message:{e.message}")
            if sql_session and in_transcation is False:
                await sql_session.rollback()
            raise
        except Exception as ext:
            hklog.error(f"service:{func.__name__} file:{func_file} error:{str(ext)}")
            if sql_session and in_transcation is False:
                await sql_session.rollback()
            raise HkBusinessException(400, "当前服务发异常") from ext
        finally:
            if sql_session and in_transcation is False:
                await sql_session.close()
            if cache_inst:
                cache_inst.close()
            if rest_client:
                await rest_client.aclose()    

            sql_session = None
            cache_inst = None
            sql_session = None
            rest_client = None
            in_transcation=False
            process_time = round(time.perf_counter() - start_time, 4)
            hklog.info(
                "leave service:"
                + func.__name__
                + " file:"
                + func_file
                + " time:"
                + str(process_time)
            )
        return result
    return wrapper



# -*- coding: utf8 -*-

"hikepy基础数据类"

__author__ = "hnck2000@126.com"
from typing import TypeVar,List,Any,Optional
from datetime import datetime
from dataclasses import dataclass,replace,asdict
from pydantic import BaseModel,Field
from sqlalchemy import DATETIME,String,CHAR,Column
from sqlalchemy.orm import DeclarativeBase

class HkBaseModel(BaseModel):
    """pydantic基础类"""

class Base(DeclarativeBase):
    """数据库模型基础类"""
    def model_dump(self):
        """将 SQLAlchemy 模型实例转换为字典"""
        return {column.name: getattr(self, column.name) for column in self.__table__.columns}

class MogoModel(BaseModel):
    """mogo基础类"""
    id:str=Field(min_length=1,max_length=64,alias="_id")
    create_user:Optional[str]=Field(default=None,min_length=1,max_length=50)
    update_user:Optional[str]=Field(default=None,min_length=1,max_length=50)
    is_delete:str=Field(default="N", min_length=1,max_length=100)
    create_time:datetime=Field(default_factory=datetime.now)
    update_time:datetime=Field(default_factory=datetime.now)

    def model_dump(self):
        """重写model_dump方法，默认使用别名"""
        return super().model_dump(by_alias=True)

class SqlModel(Base):
    """所有数据库模型都需要集成此类"""
    # 定义为抽象类
    __abstract__ = True
    # 默认字段
    id = Column(String(50),primary_key=True,comment="主键")
    create_user = Column(String(50),nullable=True,comment="创建人")
    create_time = Column(DATETIME,default=datetime.now, comment="创建时间")
    update_user = Column(String(50),nullable=True, comment="更新人")
    update_time = Column(DATETIME,default=datetime.now, comment="更新时间")
    is_delete = Column(CHAR(1), default="N", comment="删除标识：N-正常 Y-已删除")

class PageList(HkBaseModel):
    """分页查询返回结果封装类"""
    page_size:int=1
    current_page:int=1
    total_size:int=0
    total_page:int=0
    datas:List[Any]=[]

    @classmethod
    def create(cls,datas:List[Any],current_page:int=1,total_size:int=0,page_size:int=1):
        """创建分页对象"""
        total_size = 0 if total_size<=0 else total_size
        page_size = 1 if page_size<=0 else page_size
        total_page=total_size//page_size if total_size % page_size == 0 else total_size//page_size+1
        return cls(datas=datas,current_page=current_page,total_size=total_size,page_size=page_size,total_page=total_page)


Self = TypeVar('Self',bound='HkBaseData')
@dataclass
class HkBaseData:
    """基础数据类"""

    def copy(self,data:Self)->Self:
        """替换数据属性返回新的实例，如果源数据没有对于属性则忽略"""
        if data:
            if isinstance(data, HkBaseData):
                return replace(self, **data.__dict__)
            else:
                return replace(self, **data)
        return self

    def merge(self,data:Self)->Self:
        """合并两个对象"""
        if data:
            data_org=asdict(self)
            data_dest=asdict(data) if isinstance(data, HkBaseData) else data
            new_data={**data_org, **data_dest}
            return HkBaseData(**new_data)
        return self

class AuthInfo(HkBaseModel):
    """jwt用户信息"""
    user_id:str
    roles:List[str]=[]
    acts:List[str]=[]
    data:dict={}

class CommonModel(HkBaseModel):
    """通用参数类"""
    keyword:str=Field(default="",description="关键字")
    page_size:int=Field(default=20,description="每页数量")
    current_page:int=Field(default=1,description="当前页码")
    order_by:str=Field(default="",description="排序字段")
    
